#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Author: Mazhugin Aleksey
# Created: 2007/09/25
# ID: $Id: __init__.py 24 2008-12-01 17:51:12Z alex $
# URL: $URL: file:///myfiles/svn_/XPyLIB/trunc/XPyLIB/__init__.py $
# Copyright: Copyright (c) 2007, Mazhugin Aleksey
# License: BSD
#--------------------------------------------------------------------
# Package initialisation

"""
Some libraries for python programmers.

Common description.
===================
    
    XPyLIb library commonly configured throw configuration file. It's sitax see below.
    
    Root package XPyLIB do main initialization and provide some useful things.
    The B{dir_...} variables contanes path to place that they named. It more better use
    this variables then calculate path youself.
    Also you can obtain needed path by using function L{dir_get}().
    
    User configurations, logs and other things are plased in user home "~/.xpylib/" directory.

X{XPyLIB compact distributing}.
===============================
    
    You can use/distribute XPyLIB package from XPyLIB library in compact form.
    For this you need compress and move all needed files (".py" or ".pyc" or ".pyo"),
    exclude "__init__.py[co]",
    in "XPyLIB/XPyLIB" folder into "xpylib.zip" zip archive in this folder.
    In initialization XPyLIB package try find "xlogging.py[co]" and if not find
    then add to C{__path__} "xpylib.zip" path.

X{XPyLIB configuration file}.
=============================
    
    XPyLIB package auto configures from "XPyLIB/config/xpylib.cfg" file.
    This file may have next structure:
        
        - B{[Loggers]} - Loggers section:
            
            - As options may be used any names and it's values must be
                path to loggers configuration files (see L{xlogging}). There
                are four preconfigured files for XPyLIB logging:
                    
                    - B{loggers.cfg} - used for release logging - errors, warnings.
                    - B{loggers_no.cfg} - no logging output will be doing.
                    - B{loggers_trace.cfg} - logging only trace output, useful
                        for prerelease version, beta, for example.
                    - B{loggers_dbg.cfg} - logging debug and debugtrace output,
                        useful for debuging and development.
                        
        - B{[Attributes]} - Attributes for XPyLIB package:
            
            - This attributes will be readed and assign to XPyLIB package.
        
    All logging output from XPyLIB library saved into "XPyLIB/logs" folder
    if not set other. If in library occurs error then you may look logs and
    if they contains info may send it to author in zipped format.


@var _mode: Mode for folder creation.
@type _mode: int {0755}

@var __version__: XPyLIB library version in format "main.minor.svn".
@type __version__: str

@var dir_root: Absolute path to XPyLIB library.
@type dir_root: str

@var dir_rootuser: Absolute path to ".xpylib" folder in user directory. Used for
user configuration, logs and etc.
@type dir_rootuser: str

@var dir_logs: Absolute path to user ".xpylib/logs" folder.
@type dir_logs: str

@var dir_config: Absolute path to XPyLIB library "config" folder.
@type dir_config: str

@var dir_configuser: Absolute path to user configuration ".xpylib/config" folder.
@type dir_configuser: str

@var dir_xpylib: Absolute path to "XPyLIB" folder (default same as L{dir_root}).
@type dir_xpylib: str

@var log_levelbodychar: Character that fill indentation spaces.
@type log_levelbodychar: str
@var log_levelendchar: Character that insert before string output but after
    level indentation.
@type log_levelendchar: str
@var log_levelwidth: Number of indentation characters in one indentation level.
@type log_levelwidth: int
@var log_levelerr: Error message then level indentation is invalid thrn return from
    wrapped function.
@type log_levelerr: int
@var log_descfuncall: Function call description.
@type log_descfuncall: str
@var log_descfunret: Function return description.
@type log_descfunret: str

"""

import os
import sys
import ConfigParser

_mode = 755 ##mode for folder creation rwxr-xr-x

# Temporary variables.
#=====================

_v = _o = None


# Basic configuration.
#=====================

__version__ = '0.1.1'

dir_root = os.path.dirname(os.path.abspath(__file__))

dir_rootuser = os.path.join(os.path.expanduser("~"), ".xpylib")
if not os.path.exists(dir_rootuser):
    try:
        os.makedirs(dir_rootuser, _mode)
    except:
        dir_rootuser = ''
        sys.stderr.write('XPyLIB: error - can not obtane user home directory.')

def dir_get(dirpath='', user=True, create=False):
    """
    Return quering path.
    
    @param dirpath: Relative quering directory path.
    @type dirpath: str = '' or list of str ['']
    @param user: If true then return path to dir in user home dir "~/.xpylib/"
        otherwise in XPyLIB directory "/XPyLIB/".
    @type user: bool = True
    @param create: If True and path not exists then create directory recursively.
        If false then do nothing. Create new directory available only for user home dir
        (parameter L{user}=True ).
    @type create: bool = False
    @return: Absolute path to queried folder or empty string '' if no such folder or
        can't create or error occurs.
    """
    if user:
        p = dir_rootuser
    else:
        p = dir_root
        create = False
    if type(dirpath) == str:
        dirpath = [dirpath]
    p = os.path.join(p, *dirpath)
    if not os.path.exists(p):
        if create:
            try:
                os.makedirs(p, _mode)
            except:
                p = ''
        else:
            p = ''
    ##if not 
    return p

dir_logs = dir_get('logs', True, True)

dir_config = dir_get('config', False, False)

dir_configuser = dir_get('config', True, True)

dir_xpylib = dir_root

# Try add xpylibsrc.zip to package path if no xlogging.py[co] exists.
_v = os.path.join(dir_xpylib, 'xlogging.py')
if not any([ os.path.exists(_v + _o) for _o in ('', 'c', 'o') ]):
    __path__.append( os.path.join(dir_xpylib, 'xpylib.zip') )

# Import XpyLIB modules.

import xlogging

##import dbg ##@

# Read config file.
_cp = ConfigParser.SafeConfigParser()
_cp.read([os.path.join(dir_config, 'xpylib.cfg'), os.path.join(dir_configuser, 'xpylib.cfg')])


# Configure log variables (default).
log_levelbodychar = '.'
log_levelendchar = '>'
log_levelwidth = 2
log_levelerr = '*** LEVEL ERROR = '
log_descfuncall = 'CALL '
log_descfunret = 'RET from '

# Initialize loggers.
#--------------------

##dbg.fwriteln('XPyLIB Init loggers') ##@

if _cp.has_section('Loggers'):
    for _o in _cp.options('Loggers'):
        ##dbg.fwriteln('**** XPyLIB logger=',_o) ##@
        xlogging.applyConfigFile(os.path.join(dir_config, _cp.get('Loggers',_o)))
        xlogging.applyConfigFile(os.path.join(dir_configuser, _cp.get('Loggers',_o)))
        
# Read and set attributes from config.
#-------------------------------------

##dbg.fwriteln('XPyLIB Init Attributes') ##@

if _cp.has_section('Attributes'):
    for _o in _cp.options('Attributes'):
        _v=_cp.get('Attributes',_o).strip()
        if len(_v)>0:
            if _v[0] == '\'':
                # String.
                _v=_v[1:len(_v)-1]
            else:
                # Int.
                try:
                    _v=int(_v)
                except:
                    _v=0
            try:
                setattr(sys.modules[__name__], _o, _v)
            except:
                pass



# Cleanup.
del _v
del _cp
del _o
del ConfigParser
del sys
