#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Filname: lock.py
# Author: Mazhugin Aleksey
# Created: 2008/03/02
# ID: $Id: $
# URL: $URL: $
# Copyright: Copyright (c) 2008, Mazhugin Aleksey.
# License: BSD
#--------------------------------------------------------------------

"""
Threading lock classes.

This module provide some classes for easy using threading lock and local
threading variable through class inheritance.
You can mix different base classes in you derived class.

@attention: If have no threading module then locking realy is not doing
(called functions do nothing and for local data used simple object).

@var _lock: used to serialize access to shared data structures in this module.
@type _lock: L{treading.lock} or None

@var _thrdata: used to serialize access to shared data structures in this module.
@type _thrdata: L{treading.local}() or object

"""

#!!! UNDER DEVELOPMENT
return

# Importing
#import XPyLIB.dbg as dbg ##@
import XPyLIB

##:LOGGING:start
import XPyLIB.xlogging as _xlog
from functools import wraps
_log=_xlog.getLogger(__name__)
_log.disabled=0
_log_u=_xlog.IndentFormatter.lvlup
_log_d=_xlog.IndentFormatter.lvldown
_log_r=_xlog.IndentFormatter.lvlreset
_log_w=_xlog.IndentFormatter.lvlwrap
def _log_wl(fun):
    @wraps(fun)
    def wrapper(*args, **kwds):
        return _xlog.IndentFormatter.lvlwraplog(fun,_log)(*args, **kwds)
    return wrapper
##:LOGGING:end

try:
    import thread
    import threading
except:
    thread=None

# Start Implementation

_lock = thread and threading.RLock() or None
_thrdata = thread and threading.local() or object

def _acquireLock():
    """
    Acquire the module-level lock for serializing access to shared data.

    This should be released with L{_releaseLock}().
    """
    global _lock
    if _lock:
        _lock.acquire()

def _releaseLock():
    """
    Release the module-level lock acquired by calling _acquireLock().
    """
    global _lock
    if _lock:
        _lock.release()
        
class lock_rs(object):
    """
    Static rlock class.
    
    @cvar __lock_rs: qq
    @type __lock_rs: qq
    
    """
    
    _acquireLock()
    try:
        __lock_rs = thread and threading.RLock() or None
    finally:
        _releaseLock()
        
    @staticmethod
    def acquire_rs():
        """
        Acquire thread lock.
        """
        ##dbg.fwriteln('rs.acquire') ##@
        if lock_rs.__lock_rs:
            lock_rs.__lock_rs.acquire()

    @staticmethod
    def release_rs():
        """
        Release thread lock.
        """
        ##dbg.fwriteln('rs.release') ##@
        if lock_rs.__lock_rs:
            lock_rs.__lock_rs.release()
                
class lock_rd(object):
    """
    Instance rlock class.
    """
    
    def __init__(self):
        """
        @ivar __lock_rd: qq
        @type __lock_rd: qq
        
        """
        
        self.__lock_rd = thread and threading.RLock() or None
                
    def acquire_r(self):
        """
        Acquire thread lock.
        """
        ##dbg.fwriteln('rd.acquire') ##@
        if self.__lock_rd:
            self.__lock_rd.acquire()

    def release_r(self):
        """
        Release thread lock.
        """
        ##dbg.fwriteln('rd.release') ##@
        if self.__lock_rd:
            self.__lock_rd.release()

        
                
# End Implemetation

#-DEBUG-START-------------------------------------------------------------------

def _debug():
    """
    """
    ## _log.debug('Debug message')
    pass
    
#-DEBUG-END---------------------------------------------------------------------

if __name__=='__main__':
    _debug()
