#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Author: Mazhugin Aleksey
# Created: 2007/09/25
# ID: $Id: pckgmisc.py 13 2008-11-29 18:31:31Z alex $
# URL: $URL: file:///myfiles/svn_/XPyLIB/trunc/XPyLIB/pckgmisc.py $
# Copyright: Copyright (c) 2007, Mazhugin Aleksey
# License: BSD
#--------------------------------------------------------------------

"""
Miscelaneous functions for packages.
"""

import inspect
import os

def GetObjectDocs(obj):
    """
    Return first docs strings from objects item.
    
    @param obj: Object to introspection.
        Item may be modules, classes, functions.
    @type obj:  object
    @note: Get docs for all objects, include imported.
    @return: First docs strings from objects item.
    @rtype: string
    """
    d='\n'.join([name + ' - ' + str(inspect.getdoc(value)).split('\n')[0]
        for name, value in inspect.getmembers(obj, 
        lambda m: inspect.ismodule(m) or inspect.isclass(m) or
            inspect.isfunction(m)) ])
    return d
    
def ExtendPaths(sPath, *lRelPaths):
    """
    Combine sPath to each path from relative path list.
    
    @param sPath: List with item[0] = base path.
        It's list will be extended.
        B{[in/out]} parametr.
    @type sPath: list
    @param lRelPaths: Arguments list of relative paths.
    @type lRelPaths:  *string
    @return: Combined C{sPath} to each path from relative path list in the
        B{sPath}.
    @rtype: None
    """
    sPath.extend([os.path.join(sPath[0], item) for item in lRelPaths])

def cutarch(path, arch='.zip'):
    """
    If path C{path} end with C{arch} file then remove this file and return
    path without C{arch}.
    
    For define archive test ends of path and if it equal arch then archive is defined.
    
    For example if C{path = 'c:/lib/pack.zip'} then return C{'c:/lib'}.
    
    @param path: Path.
    @type path: str
    @param arch: Archive type.
    @type arch: str = {'.zip'}
    @return: Path without archive at end.
    @rtype: str
    """
    if path.endswith(arch):
        path = os.path.dirname(path)
    return path
