#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Filname: handler_test.py
# Author: Mazhugin Aleksey
# Created: 2007/11/06
# ID: $Id: handler_test.py 13 2008-11-29 18:31:31Z alex $
# URL: $URL: file:///myfiles/svn_/XPyLIB/trunc/XPyLIB/test/handler_test.py $
# Copyright: Copyright (c) 2007, Mazhugin Aleksey.
# License: BSD
#--------------------------------------------------------------------

"""
Unit test for XPyLIB.handler.
"""

# Importing
import XPyLIB.handler as tm
import unittest
import XPyLIB.dbg as dbg ##@
import re

# Start test

class HO(object):
    """
    Handler object.
    """
    def __init__(self, i):
        self.i=i
    def fun1(self, msg=''):
        return 'fun1: ' + msg + ' ' + str(self.i)

def fun2(self, msg=''):
        return 'fun2: ' + msg
    
def fun1(msg=''):
    return msg != 'stop'
    
class EH(tm.EventFilter):
    """
    """
    def __init__(self, sit=True):
        super(EH, self).__init__(sit)
        self.log=[]
        
    def __call__(self, *args, **kwds):
        self.log.append( 'args="%s", kwds="%s"' % (str(args), str(kwds)) )
        r=super(EH, self).__call__(*args, **kwds)
        self.log.append( 'r="%s"' % (str(r), ) )
        return r
        
class TestHandler(unittest.TestCase):
    """Test handler"""

    def setUp(self):
        self.ho1=HO(1)
        self.ho2=HO(2)
        self.ho3=HO(3)
        self.eht=EH(True)
        self.ehf=EH(False)
        pass

    def tearDown(self):
          pass

    def test_general(self):
        self.to=tm.Handler()
        ##dbg.fwriteln('__str__==', str(self.to)) ##@
        # add.
        self.assertEqual(self.to.handler_add(self.ho1),0)
        self.assertEqual(self.to.handler_add(self.ho2),1)
        # Next must be skipped.
        self.assertEqual(self.to.handler_add(self.ho1),1)
        # len.
        self.assertEqual(len(self.to), 2, 'Len error.')
        # has_handler.
        self.assert_(self.to.handler_has_handler(self.ho1,self.ho3)==[0,-1], 'Has handler error')
        ##self.assert_(self.to.handler_has_handler(self.ho3)==-1, 'Find wrong handler')
        # get_handler.
        self.assertEqual(self.to.handler_getHandler(1),self.ho2, 'Can\'t get handler')
        # get attribute and function call.
        r=self.to.fun1('msg')
        ##dbg.fwriteln('r=', r) ##@
        self.assert_('fun1: msg 1' in r, 'Func call error')
        self.assert_('fun1: msg 2' in r, 'Func call error')
        self.assertEqual(len(r), 2, 'Func call error')
        # test error attribute.
        r=self.to.remove(self.ho1)
        ##dbg.fwriteln('remove=', r) ##@
        self.assert_(isinstance(r[0], tm.HandlerError), 'Must be error')
        self.assert_(isinstance(r[1], tm.HandlerError), 'Must be error')
        ##dbg.fwriteln('len=', len(self.to)) ##@
        ##dbg.fwriteln('handlers==', self.to._Handler__handler_hl) ##@
        # remove.
        self.to.handler_remove(self.ho1)
        self.assertEqual(len(self.to), 1, 'Len error.')
        self.assert_(self.to.handler_has_handler(self.ho2)==[0], 'Not find handler')
        self.assert_(self.to.handler_has_handler(self.ho1)==[-1], 'Find wrong handler')
        # add
        ##dbg.fwriteln('add') ##@
        self.assertEqual(self.to.handler_add(self.ho3),1)
        # len.
        ##dbg.fwriteln('len') ##@
        self.assertEqual(len(self.to), 2, 'Len error.')
        ##dbg.fwriteln('self.to.i._Handler__handler_hl==',self.to.i._Handler__hl) ##@
        self.assert_(self.to.i._Handler__handler_hl == [2,3], 'get attribute error')
        #del
        ##dbg.fwriteln('delete i') ##@
        del self.to.i
        ##dbg.fwriteln('self.to.i',str(self.to.i)) ##@
        self.assert_(str(self.to.i) == \
            "Handler instance: [HandlerError(): 'HO' object has no attribute 'i',"+\
            " HandlerError(): 'HO' object has no attribute 'i']", 'Must be error')
        
        self.to=tm.Handler()
        self.to.handler_add(1,2,3,4,5)
        self.assert_(self.to._Handler__handler_hl == [1,2,3,4,5], 'add error')
        self.assert_(self.to.handler_insert(2,6,7,8) == 5, 'insert error')
        self.assert_(self.to._Handler__handler_hl == [1,2,6,7,8,3,4,5], 'insert error')
        self.assert_(self.to.handler_pop(5) == 3, 'pop error')
        self.assert_(self.to._Handler__handler_hl == [1,2,6,7,8,4,5], 'pop error')
        
        pass
    
    def test_eventfilter(self):
        self.to=tm.Handler()
        self.to.handler_add(fun2)
        self.to.handler_add(fun1)
        self.to.handler_add(self.ho1.fun1)
        self.to.handler_filter = self.ehf
        r=self.to('testmessage')
        ##dbg.fwriteln('r="%s"' % (str(r), )) ##@
        tr='''['fun2: ', True, 'fun1: testmessage 1']'''
        self.assert_( tr == str(r) )
        
        r=self.to('stop')
        ##dbg.fwriteln('r="%s"' % (str(r), )) ##@
        tr='''['fun2: ', False]'''
        self.assert_( tr == str(r) )
        
        self.to.handler_filter = self.eht
        r=self.to('testmessage')
        ##dbg.fwriteln('r="%s"' % (str(r), )) ##@
        tr='''['fun2: ']'''
        self.assert_( tr == str(r) )
        
        r=self.to('stop')
        #3dbg.fwriteln('r="%s"' % (str(r), )) ##@
        tr='''['fun2: ']'''
        self.assert_( tr == str(r) )
        
        self.to.handler_remove(fun2)
        r=self.to('stop')
        ##dbg.fwriteln('r="%s"' % (str(r), )) ##@
        tr='''[False, 'fun1: stop 1']'''
        self.assert_( tr == str(r) )
        
        r=self.to('testmessage')
        ##dbg.fwriteln('r="%s"' % (str(r), )) ##@
        tr='''[True]'''
        self.assert_( tr == str(r) )
        
        pass


# End test

if __name__=='__main__':
    unittest.main(argv=['','-v']) # Verbosity.
