#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Author: Mazhugin Aleksey
# Created: 2007/09/25
# ID: $Id: codetpl.py 20 2008-11-29 18:40:06Z alex $
# URL: $URL: file:///myfiles/svn_/XPyLIB/trunc/XPyTools/codetpl/codetpl.py $
# Copyright: Copyright (c) 2007, Mazhugin Aleksey
# License: BSD
#--------------------------------------------------------------------

"""
Main code template module.

Overview.
=========
    
    Provide core functionality for code templates.
    
    Tunning of this module may be prformed by using configuration files.
    Default file is "I{codetpl_default.cfg}" that must not be changed.
    User preferences may be added to custom configuration files named
    "I{codetpl.cfg}" . Search for custom configuration files in all
    templates folder and combined and override settings from las files.
    
    Module have next objects:
        - Exceptions: L{codetplError}, L{codetplArgumentError},
            L{codetplTemplateException}.
        - Main class L{CodeTemplateMngr} that provide core manager for templates.
        - Variable L{ANY} value of which is equivalent any other value in
            metainformation of templates.

X{CodeTemplateMngr overview}.
=============================
    
    Use L{CodeTemplateMngr.HasType} to test type for exists.
    Use L{CodeTemplateMngr.GetTemplates} to get available templates list.
    Use L{CodeTemplateMngr.MetaNames}, L{CodeTemplateMngr.GetMetaInfo},
    L{CodeTemplateMngr.GetMetaAllValues} to get metainformation about
    templates.
    Use L{CodeTemplateMngr.Types} to get available types of templates.
    Use L{CodeTemplateMngr.GetConfig} to get preferences from
    configuration files.
    Use L{CodeTemplateMngr.ParseTemplate} to parse templates.
    
    You can reload module for changest in files make effect by calling
    L{CodeTemplateMngr.Reload}.

X{Configuration file (CodeTemplateMngr)}.
=========================================
    
    The user configuration file example (it's get from codetpl_default)::
    
        #!/usr/bin/env python
        # -*- coding: utf-8 -*-
        #--------------------------------------------------------------------
        # Author: Mazhugin Aleksey
        # Created: 2007/10/24
        # ID: $Id: codetpl.py 20 2008-11-29 18:40:06Z alex $
        # URL: $URL: file:///myfiles/svn_/XPyLIB/trunc/XPyTools/codetpl/codetpl.py $
        # Copyright: Copyright (c) 2007, Mazhugin Aleksey
        # License: BSD
        #--------------------------------------------------------------------
        
        # Default settings for Code templates.
        #
        # This config file for users folder. 
        # You may copy and rename it to you home
        # "~/.xpylib/xpytools/codetpl/templates/codetpl.cfg"
        # and edit configuration.
        
        [Prefs]
        
        # If true then will be loaded global templates from
        # "XPyLIB/XPyTools/codetpl/templates/".
        # If False when will be used only user templates.
        UseGlobalTemplates = yes
        
        #If true then reload templates files every tyme, else only
        # at startup. Also with templates each time reload configuration
        # file.
        ReloadTemplates = yes
        
        # Templates types.
        # Preferences for each type see in [type_???] section.
        # If type is not included then it will be not used.
        types = act, new, wzd, kwd, tpl, wtp
        
        # Prefix for types sections.
        # Types sectins will be search into typesprefix??? where ???-type.
        typesprefix = type_
        
        # Meta information fields names.
        MetaNames = category, language, author
        
        # Any value equivalent
        ANY = any
        
        #If not empty then add to Tools menu with it name.
        # TODO: Not releazed yet, add.
        ToolMenuName = Code Template
        
        [Template_paths]
        
        ################################################################################
        #Template paths.
        ################################################################################
        
        # Default path - same dir as config file.
        path1 = .
        
        # Additional paths.
        #path2 = 
        #path3 = 
        
        
        
        ################################################################################
        # Types settings
        ################################################################################
        #
        # Name - Full name for template.
        #
        # Desc - Description for given type.
        #
        # HotKey - Hot key for call teplates from editor.
        #
        # MenuName - If exist and not empty then register in to menu Edit->... .
        #
        # Interface - Interface of template type. May be:
        #   - 'gui' - GUI interface (used by default).
        #   - 'list' - Standard editor list popup.
        #
        
        [type_act]
        Name = Action
        Desc = Action for code manipulation
        HotKey = Alt+A
        Interface = list
        MenuName = Code Template Action
        
        [type_kwd]
        Name = Keywords
        Desc = Any keywords
        HotKey = Alt+K
        Interface = list
        MenuName = Code Template Keyword
        
        [type_tpl]
        Name = Code
        Desc = Code templates
        HotKey = Alt+T
        Interface = list
        MenuName = Code Template Template
        
        [type_wzd]
        Name = Wizard
        Desc = Wizard for code creation.
        HotKey = 
        Interface = gui
        MenuName = Code Template Wizard
        
        [type_new]
        Name = New
        Desc = Templates for new files
        HotKey = Alt+Q
        Interface = gui
        MenuName = Code Tpl New
        
        [type_wtp]
        Name = XTemplate
        Desc = Extended code templates.
        HotKey =
        Interface = gui
        MenuName = Code Tpl Tpls wizard

@var ANY: Value that equivalent any other value (used in metainformation).
@type ANY: str

"""

import sys
import os
import imp
import inspect
import glob
import re
import ConfigParser
import XPyLIB
##import XPyLIB.dbg as dbg ##@
    
# var: Value that equivalent any other value (used in metainformation).
# type: str
ANY = 'any'

class codetplError(Exception):
    """
    Code templates common exception.
    """
    pass

class codetplArgumentError(codetplError):
    """
    Code templates argument exception.
    """
    pass

class codetplTemplateException(codetplError):
    """
    Exception then executing template function.
    
    Argument Err is Exception object.
    """
    pass

class CodeTemplateMngr(object):
    """
    Manager of code templates.
    
    Provide all functionalities with templates.
    """
    
    def __get_Errors(self):
        return self.__errs[:]
    
    def __del_Errors(self):
        self.__errs=[]
          
    #: @ivar: List of errors string (copy, not original).
    #: @type: list
    Errors = property(__get_Errors, None, __del_Errors)
    
    @property
    def Types(self):
        """
        List of available templates types (copy, not original).
        Type - list.
        """
        return self.__types[:]
    
    @property
    def MetaNames(self):
        """
        List of available metanames (copy, not original).
        Type - list.
        """
        return self.__metanames[:]
    
    @property
    def SourceFile(self):
        """
        Source file which editing in the editor.
        Type - str = C{''}
        """
        return self.__srcfile
    
    def __init__(self, spath = [], usedefault = True):
        """
        Initialize class.
        
        @param spath: List of absolute paths to templates folders. If empty string
        (or C{False}) then set default path == "... /codetpl/templates/"
        @type spath: list[string] = C{[]}
        @param usedefault: If true then load global and user config files and
            if allow use global templates. Otherwise use only path from L{spath}.
        @type usedefault: bool = True
        @raise codetplArgumentError: Template path does not exist. May raise
            for default path or user specified.
        @warning: Into initialization folder must be placed configuration
            file B{C{codetpl.cfg}} else wil be used default values (see 
            module description).
        """
        
        #: @ivar: Edited source file, from which called templates.
        #: @type: string = C{''}
        self.__srcfile='' 
        
        #: @ivar: C{SafeConfigParser} instance.
        #: @type: SafeConfigParser
        self.__config=None
        
        #: @ivar: Loaded module's names.
        #: @type: list = C{[]}
        self.__mdls=[]  
        
        #: @ivar: Keywords functions readed from files:
        #:  {"FullMame=???_*",Code_object}.
        #: @type: dict = C{{}}
        self.__kwds={}   
        
        #: @ivar: Other templates.
        #:  {"FullMame=???_*" : func_object}
        #: @type: dict = C{{}}
        self.__tpls={}
        
        #: @ivar: Templates meta (all for keywords and other).
        #:  { "FullMame=???_*" : {"metaname" : "metavalue"} }
        #: @type: dict = C{{}}
        self.__tplsmeta={}
        
        #: @ivar: Errors list. You must clear it's self.
        #:  Auto clear then L{Reload() <Reload>} calls.
        #: @type: list = C{[]}
        self.__errs=[]
        
        #: @ivar: Available types for templates.
        #: @type: list = C{[]}
        self.__types=[] 
        
        if type(spath) == str:
            spath=[spath]
        #: @ivar: List paths to templates.
        #: @type: list[str]
        #:  paths to folders with templates.
        self.__path = spath 
        
        #: @ivar: If true then load global and user config files and
        #:     if allow use global templates. Otherwise use only path from L{spath}.
        #: @type: bool = True
        self.__usedefault = usedefault
            
        #: @ivar: Names for available fields of metainformation (must present).
        #: @type: list
        self.__metanames=['category', 'language']
        
        #: @ivar: Regular exspression for metainformation fields (must present).
        #: @type: list
        self.__metare=[ re.compile(r'^#\s*' + n + r'\s*=\s*(?P<' + \
            n + r'>.*)$', re.M) for n in self.__metanames ]

        self.Reload()
        
    def GetConfig(self, section, option=None, default=''):
        """
        Return preferences from configuration files.
        
        You may get option value or section items:
            - Option.
                If option not exist then return default value.
            - Section.
                For get section items you must leave I{option} parameter
                empty or set to C{None}.
                If no such section then return empty list [].
        
        @param section: Section name.
        @type section: str
        @param option: Option name.
        @type option: str=C{''}
        @param default: Default value.
        @type default: str=C{''}
        @return: Configuration value or list of items for section.
        @rtype: str or list[(name, value)]
        """
        if option == None:
            return self.__config.has_section(section) and \
                self.__config.items(section) or []
        else:
            return self.__config.has_section(section) and \
                self.__config.has_option(section, option) and \
                self.__config.get(section, option) or default
    
    def Reload(self):
        """
        Re/Load template and config modules.
        
        Clear previous loaded modules and reload all noe finded.
        All loaded template modules into __mdls, naming is as "_codetpl_???_*".
        Walk throw all loaded modules and fill __kwds and __tpls dictionaries.
        
        @return: None.
        @rtype: None
        """
        ##dbg.fwriteln('codetpl.Reload()') ##@
        # Clear line cache from inspect module.
        inspect.linecache.clearcache()
        
        # Clear all.
        self.__kwds.clear()
        self.__tpls.clear()
        self.__errs=[]
        self.__types=[]
        self.__tplsmeta.clear()

        self.__config = ConfigParser.SafeConfigParser()
        
        if self.__usedefault:
            # Read default config.
            defcfgpath = os.path.join( os.path.dirname(__file__), \
                'templates', 'codetpl_default.cfg')
            if os.path.exists(defcfgpath):
                # Read default.
                try:
                    ## dbg.writeln('\t load defcfg==', defcfgpath) ##@
                    fp=open(defcfgpath)
                    self.__config.readfp(fp)
                except:
                    ## dbg.writeln('\t load defcfg==failed', defcfgpath) ##@
                    pass # Be quit.
                finally:
                    if fp:
                        fp.close()
                        
            # Read user config.
            defcfgpath = XPyLIB.dir_get('xpytools/codetpl/templates', True, True)
            defcfgpath = os.path.join( defcfgpath , 'codetpl.cfg')
            if os.path.exists(defcfgpath):
                # Read user config.
                try:
                    ## dbg.writeln('\t load defcfg==', defcfgpath) ##@
                    fp=open(defcfgpath)
                    self.__config.readfp(fp)
                except:
                    ## dbg.writeln('\t load defcfg==failed', defcfgpath) ##@
                    pass # Be quit.
                finally:
                    if fp:
                        fp.close()
            
            if self.GetConfig('Prefs','UseGlobalTemplates','yes') == 'yes':
                self.__path.append( os.path.join( os.path.dirname(__file__), 'templates') )
        ##if self.__usedefault
        
        # Load configs from paths.
        defcfgpath = [os.path.join( p, 'codetpl.cfg')
            for p in self.__path] 
        ## dbg.writeln('\t load ini cfg==', defcfgpath) ##@
        self.__config.read(defcfgpath)
        
        # Get config available types.
        availtypes=[n.strip() for n in self.GetConfig('Prefs','types','').split(',')]
        global ANY
        ANY=self.GetConfig('Prefs','ANY','any')
        # Get config metanames.
        mns=[n.strip() for n in self.GetConfig('Prefs','MetaNames','').split(',')]
        for mn in mns:
            if mn not in self.__metanames:
                self.__metanames.append(mn)
                self.__metare.append( re.compile(r'^#\s*' + mn + r'\s*=\s*(?P<' + \
            mn + r'>.*)$', re.M) )     
        ##end for mn
        
        ## dbg.writeln('\n\tavailabletypes==',availtypes) ##@
        
        ##dbg.fwriteln('codetpl.Reload() delete modules:', self.__mdls) ##@
        for f in self.__mdls:
            try:
                ##dbg.fwriteln('codetpl.Reload() try pop: ', str(f)) ##@
                m=sys.modules.pop('_codetpl_' + f, 1) ## 0 - for quet work if no module.
            except Exception, e:
                ##dbg.fwriteln('codetpl.Reload() except pop: ', str(f)) ##@
                pass
            del m
            ##dbg.fwriteln('codetpl.Reload() delete result:', ('_codetpl_' + f) not in sys.modules) ##@
            
        ## dbg.writeln([n for n in sys.modules.keys() if n.startswith('_')]) ##@
        self.__mdls=[]
        
        # Loop folders
        cfgpath=[v for k,v in self.GetConfig('Template_paths')]
        ## dbg.writeln('\t load folders==', cfgpath) ##@
        ## dbg.writeln('\t folders paths==', self.__path + cfgpath) ##@
        for folder in self.__path + cfgpath:
            ## dbg.writeln('\t *** folder==', folder) ##@
            if not os.path.exists(folder):
                ## dbg.writeln('\t *** *** not exists', folder) ##@
                continue
            # Loop for files into templates directory
            ## dbg.writeln( 'glob=', os.path.join(folder,'???_*.py') ) ##@
            for f in glob.iglob(os.path.join(folder,'???_*.py')):
                ## dbg.writeln('\t *** f==', f) ##@
                if not os.path.isdir(f):
                    # Create and add module name to list, load module.
                    # Save all names into lowercase.
                    n=os.path.splitext(os.path.basename(f))[0].lower()
                    pref=n[0:3] ## Template type.
                    # Load only types available by config.
                    # dbg.writeln('\n\tpref===',pref, ' n=', n) ##@
                    if pref in availtypes:
                        try:
                            ## dbg.writeln( 'try load=', '_codetpl_' + n,'==', f ) ##@
                            module = imp.load_source('_codetpl_' + n, f)
                        except Exception, error:
                            # Log error and quet pass.
                            ## dbg.writeln( '  try except' ) ##@
                            self.__errs.append(str(error) + \
                                '. (' + str(error.__class__) + ').')
                        else:
                            #If all OK'ey then continue.
                            ## dbg.writeln( '  try else' ) ##@
## You may uncomment this for prevent loading modules with equals names.
##                            if n in self.__mdls:
##                                # Except loadind twice modules with like names.
##                                continue
                            self.__mdls.append(n)
                            # Import templates.
                            prefu=pref + '_'
                            for name, val in inspect.getmembers(module, \
                            lambda m: inspect.isfunction(m) and \
                                m.__name__.lower().startswith(prefu) ):
                                # Add function.
                                name=name.lower()
                                npref=name[0:3]
                                # Get metainformation.
                                ##vcmnts=str(inspect.getcomments(val))
                                vcmnts=unicode(inspect.getcomments(val),errors='replace') #:PATCH: Unicode.
                                # Documentation string as description.
                                ##vdoc=str(inspect.getdoc(val))
                                vdoc=unicode(inspect.getdoc(val),errors='replace') #:PATCH: Unicode.
                                if npref not in self.__types:
                                    # Add types if new
                                    self.__types.append(npref)
                                # Add meta info (desc==docstring,{metaname:metavalue}).
                                md=[]
                                for i in range(len(self.__metanames)):
                                    n=self.__metanames[i]
                                    ## dbg.writeln('\n\t\t Search metanames==',n) ##@
                                    m=self.__metare[i].search(vcmnts)
                                    if m:
                                        ## dbg.writeln('\n\t\t append metanames==',n, ' ** ', str(m.group(n)).strip()) ##@
                                        md.append( (n, str(m.group(n)).strip() or '') )
                                ##end for n
                                self.__tplsmeta[name]=( vdoc, dict(md) )
                                ## dbg.writeln('\n\t\t ADD META(',name,'==',( vdoc, dict(md) )) ##@
                                if npref=='kwd':
                                    # Add to keywords without type prefix.
                                    self.__kwds[name[4:]]=val
                                else:
                                    # Add to templates.
                                    self.__tpls[name]=val
                            ##end if
                        ##end for
                    ##end try
                ##end if
            ##end for glob
        ##end for folder
    
    def GetTemplates(self, type_, filter={}, retIfAbsent=False):
        """
        Get list of available templates given type.
        
        C{type} must be one from L{C{Types()}<Types>}.
        If no such type then raise L{codetplArgumentError}.
        
        C{type} are case insensetive (converts to lower).
        
        Filter contains dictionary {name:listofvalues} where listofvalues
        may be tuple, list or other iterable or str (converted to single tuple).
        Filter do only from given metanames other is always done.
        
        @param type_: Type of templates.
        @type type_: str
        @param filter: Filter from metainformation fields.
        @type filter: dict=C{empty}
        @param retIfAbsent: If metanames from filter absent in templates then
        if C{True} return this template, if C{False} then skip.
        @type retIfAbsent: boolean=C{False}
        @return: List of templates name for given C{type}.
        @rtype: list
        @raise codetplArgumentError: If no such type of template.
        """
        type_=type_.lower()
        if not self.HasType(type_):
            ##raise codetplArgumentError, 'Unknown template type:' + type_ +'.'
            # Templates not found, write to stderr.
            if type_ == '':
                em = 'CodeTemplates: Templates not found.'
            else:
                em = 'CodeTemplates: Unknown template type: "' + type_ +'".'
            sys.stderr.write(em)
            # Return empty list
            return []
        pref = type_ + '_'
        if type_=='kwd':
            tpls = self.__kwds.keys()
        else:
            # From names escaped "???_".
            tpls = [name[4:] for name in self.__tpls.keys()
                if name.startswith(pref)]
        # Filter templates.
        ret=[]
        for n in tpls:
            if filter:
                # Convert to secuense.
                for k,v in filter.items():
                    if type(v) == str:
                        v=(v,)
                    if self.__tplsmeta[pref+n][1].has_key(k):
                        # Test key values.
                        vv=self.__tplsmeta[pref+n][1][k]
                        if (ANY in v) or (vv in v) or (vv==ANY):
                            # Append if value right.
                            ret.append(n)
                    else:
                        # No such key, try absent.
                        if retIfAbsent:
                            ret.append(n)
                    ##end if
                ##end for k,v
            else:
                ret.append(n)
            ##end if filter
        ##end for n
        
        ## dbg.writeln('\n    TPLS==', tpls, '\n      RET==',ret) ##@
        
        return ret
        
    def GetMetaInfo(self, type_, name):
        """
        Get metainformation for requested template.
        
        If C{type} or C{name} is absant then return C{I{ ('',{}) }}
        
        @warning: Returned value is not copy, it saved object. If you
        change it you change metainformation.
        
        @param type_: Template type.
        @type type_: str
        @param name: Template name.
        @type str
        @return: Tuple of C{str} and C{dict}
            ('docstr', {'metaname' : 'metavalue'})
        @rtype: tuple
        """
        type_=type_.lower()
        name=name.lower()
        n = type_ + '_' + name
        ## dbg.writeln('\n\t**** GETMETA==',type_+' ', name+' ', n+' ', self.__tplsmeta[n]) ##@
        return ( (type_ in self.__types) and self.__tplsmeta.has_key(n) \
            and self.__tplsmeta[n] ) or ('',{})
    
    def GetMetaAllValues(self, type_, metaname):
        """Get list of all values find in given C{type} for given C{metaname}.
        
        @param type_: Template type.
        @type type_: str
        @param metaname: Metaname.
        @type str
        @return: List of all values find in given C{type} for given C{metaname}.
        @rtype: list
        """
        pref=type_.lower() + '_'
        metaname=metaname.lower()
        ret=[]
        for k,v in self.__tplsmeta.iteritems():
            if v[1].has_key(metaname):
                vv=v[1][metaname]
                if ret.count(vv)==0:
                    ret.append(vv)
        return ret
    
    def ParseTemplate(self, type, name, fields, srcfile='', params={}):
        """
        Parse template used given fields or return preview.
        
        C{type} and C{name} are case insensetive (converts to lower).
        
        If you get empty L{fields} then for all finded keywords will be
        assigned it's name. It's used for preview template. For real parse
        you may set into fields neded values.
        
        @param type: Template type. See L{Types}.
        @type type: str
        @param name: Template name.
        @type name: str
        @param fields: Fields of keywods: {"name":"value"}. Changed by this
            function to finded keywords.
        @type fields: dict
        @param srcfile: Source file, which editing into editor.
        @type srcfile: str =C{''}
        @param params: Additional parameters. (Usually for boa constructor its
            'model' and 'view' objects).
        @type params: dict = {}
        @return: Parsed template, and changed fields (added new).
        @rtype: str
        """
        
        if not isinstance(fields, dict):
            raise codetplArgumentError, 'Argument "fields" must be a dict.'
        if not self.HasType(type):
            raise codetplArgumentError, 'Unknown template type.'
        self.__srcfile=srcfile
        ##dbg.writeln('\n\tSRCFILE assign==', self.__srcfile) ##@
        type=type.lower()
        name=name.lower()
        if type=='kwd':
            d=self.__kwds
            n=name
        else:
            d=self.__tpls
            n=type + '_' + name
        if not d.has_key(n):
            raise codetplArgumentError, 'No such template "' + n + '".' 
        # Safe call of template function.
        try:
            sys.modules[d[n].__module__].codetpl_sourcefile=self.__srcfile
            sys.modules[d[n].__module__].codetpl_params=params
            ## dbg.writeln('\n    module dict== ', sys.modules[d[n].__module__].__dict__['codetpl_sourcefile']) ##@
            if type=='kwd':
                tpl=d[n]() ## Must return str.
            else:
                tpl=d[n](fields) ## Must return str and changed or new fields.
        except Exception, err:
            e=codetplTemplateException('Exception in template: ' + str(err))
            e.Err=err
            raise e
        # If keywords then do not parse.
        if type=='kwd':
            return str(tpl)
        # Use self.__kwds, borders is '@', case insensetive.
        tpl=self.ReplaceTemplate(tpl, fields, None, '@', False, srcfile, params)
        return tpl
        
    def ReplaceTemplate(self, tpl, fields, kwds=None, brds='@', case=False, \
            srcfile='', params={}):
        """
        Parse template C{tpl}. Change all C{fields} rounded borders
        C{brd} to value of C{fields}. If fields not find then try
        search into C{kwds}. If not find then use fields key as value.
        
        @param tpl: Template string to parse with fields.
        @type tpl: str
        @param fields: Fields for search and replace into C{tpl}.
        @type fields: dict
        @param kwds: Keywords for search if not find into C{fields}.
            - If C{None} then use instance L{__kwds}.
            - If dict then its value may be:
                - str - use directly.
                - function - call with no arguments, must return string. 
        @type kwds: B{None}, str or dict{key: str or func()}
        @param brds: Borders wrapped around keys.
            Twice brds is parsed as single character 'brds'.
        @type brds: str = '@'
        @param case: If True then keys is casesensetive.
        @type case: boolean = False
        @param srcfile: Source file, which editing into editor.
        @type srcfile: str =C{''}
        @param params: Additional parameters. (Usually for boa constructor its
            'model' and 'view' objects).
        @type params: dict = {}
        @return: Replaced template, and changed fields (added new).
        @rtype: str
        """
        
        self.__srcfile=srcfile
        # Pattern to match keys into templates.
        if case:
            flags=re.I | re.S
        else:
            flags=re.S
        self.__ro=re.compile(r'(?<!'+brds+r')'+brds+r'\w+?' \
            +brds+r'(?!'+brds+r')', flags)
        
        if kwds==None:
            kwds=self.__kwds
        # Convert keys to lower.
        d=dict( [ (k.lower(), v) for k,v in fields.iteritems() ] )
        dk=dict( [ (k.lower(), v) for k,v in kwds.iteritems() ] )
        # Find all @???@ and plase into dict F.
        ##tpl=str(tpl) # Only ANSY support.
        tpl=unicode(tpl,errors='replace') #:PATCH: Unicode support.
        ## dbg.writeln('-------FILL FIELDS-------') ##@
        
        #Removing "@" from sides and make lower.
        F=[ k[1:-1] for k in self.__ro.findall(tpl) ]
        ## dbg.writeln('F=%s' % (str(F),) ) ##@
        ## dbg.writeln('d=%s' % (str(d),) ) ##@
        ## dbg.writeln('dk=%s' % (str(dk),) ) ##@
        ## dbg.writeln('kwds=%s' % (str(kwds),) ) ##@
        
        # Calculate all F.
        f={}
        for kk in F:
            k=kk.lower()
            ## dbg.writeln('    try find field=%s' % (k,) ) ##@
            if d.has_key(k):
                ##r=str(d[k]) # Only ANSY support.
                r=unicode(d[k]) #:PATCH: Unicode support
                ## dbg.writeln('        find field %s=%s' % (k,r) ) ##@
                ## dbg.write('        dict keyword %s=' % (str(dk),) ) ##@
            elif dk.has_key(k):
                ## dbg.write('        find keyword %s=' % (k,) ) ##@
                if callable(dk[k]):
                    sys.modules[dk[k].__module__].codetpl_sourcefile=self.__srcfile
                    sys.modules[dk[k].__module__].codetpl_params=params
                    ## dbg.writeln( 'Module srcfile==', sys.modules[dk[k].__module__].codetpl_sourcefile) ##@
                    ## dbg.writeln( 'Instance srcfile==', self.__srcfile) ##@
                    ##r=str(dk[k]()) # Only ANSY support.
                    r=unicode(dk[k]()) #:PATCH: Unicode support.
                    ## dbg.writeln('func(%s)=%s' % (dk[k].__name__,r) ) ##@
                else:
                    r=str(dk[k])
                    ## dbg.writeln('%s' % (r,) ) ##@
            else:
                r=kk
                ## dbg.writeln('        No such field, keyword') ##@
            f[k]=r
            fields[kk]=r
            ## dbg.writeln('  *** %s=%s' % (k,f[k]) ) ##@
            
        # Replace into tpl all F.
        r=tpl
        ro=re.compile(r'@@',re.I | re.S)
        r=ro.sub('@',r)
        for k in f:
            ro=re.compile(r'(?<!@)@'+k+r'@(?!@)',re.I | re.S)
            r=ro.sub(f[k],r)
        # Remove first and last empty lines.
        r=r.splitlines(True)
        if r[0].strip()=='':
            r.pop(0)
        if r[-1].strip()=='':
            r.pop(-1)
        return ''.join(r)
        
    def HasType(self, type_):
        """
        Test - has such type or not.
        
        It's more perfomance then use I{C{'type' in Types}}, becouse
        it use inner fields, then return copy.
        
        C{type} are case insensetive (converts to lower).
        
        @param type_: Tested template type.
        @type type_: str
        @return: C{True} if has, else C{False}.
        @rtype: boolean.
        """
        type_=type_.lower()
        return type_ in self.__types
        
    def GetSourceFile(self):
        """
        Get edited surce file.
        
        @return: Source file path or C{''} if no.
        @rtype: string
        """
        return self.__srcfile




#-DEBUG-START-------------------------------------------------------------------

def _debug():
    """
    """
    pass
    
#-DEBUG-END---------------------------------------------------------------------

if __name__=='__main__':
    _debug()
    

