#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Author: Mazhugin Aleksey
# Created: 2007/09/24
# ID: $Id: kwd_keywords.py 7 2007-11-15 18:12:30Z alex $
# URL: $URL: file:///H:/svn_/XPyLIB/trunc/XPyTools/codetpl/templates/kwd_keywords.py $
# Copyright: Copyright (c) 2007, Mazhugin Aleksey
# License: BSD
#--------------------------------------------------------------------

"""
Provide keywords for replace into code templates.
        
For retrive source file path you must use C{codetpl_sourcefile} global
variable which auto appending into module dictionary.

@note: Functions must have prefix "kwd_".

Global variable C{codetpl_sourcefile} added from caller and containce
source full path.
"""

import datetime as dt
import getpass
import os

# category = general
# language = any
# author = Mazhugin Aleksey
def kwd_date():
    """
    Return current date into format: YYYY/MM/DD.
    """
    return dt.date.today().strftime('%Y/%m/%d')

# category = general
# language = any
# author = Mazhugin Aleksey
def kwd_year():
    """
    Return current year in format YYYY.
    """
    return dt.date.today().strftime('%Y')

# category = general
# language = any
# author = Mazhugin Aleksey
def kwd_license():
    """
    Return license type that you provided
    """
    lic='''BSD'''
    return lic

# category = general
# language = any
# author = Mazhugin Aleksey
def kwd_licensetext():
    """
    Return license text that you provided
    """
    lictxt='''# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the license
# file for more details.'''
    return lictxt

# category = general
# language = any
# author = Mazhugin Aleksey
def kwd_author():
    """
    Return author.
    
    By default get current user and return it.
    If user undefined return "Unknown".
    """
    user=getpass.getuser() or 'Unknown'
    return user

# category = general
# language = any
# author = Mazhugin Aleksey
def kwd_copyright():
    """
    Return copyrights
    
    By default return: "Copyright (c) YYYY, author."
    """
    cr='Copyright (c) ' + dt.date.today().strftime('%Y') + ', ' + kwd_author() + '.'
    return cr

# category = general
# language = any
# author = Mazhugin Aleksey
def kwd_filename():
    """
    Return source file name with extension.
    """
    return os.path.basename(codetpl_sourcefile)

# category = general
# language = any
# author = Mazhugin Aleksey
def kwd_basefilename():
    """
    Return source base file name without extension.
    """
    return os.path.splitext(os.path.basename(codetpl_sourcefile))[0]

# category = general
# language = any
# author = Mazhugin Aleksey    
def kwd_filenamefull():
    """
    Return full source file name (full path and file name).
    """
    return codetpl_sourcefile

# category = general
# language = any
# author = Mazhugin Aleksey    
def kwd_filepath():
    """
    Return path to source file.
    """
    return os.path.split(codetpl_sourcefile)[0]

# category = general
# language = any
# author = Mazhugin Aleksey    
def kwd_fileext():
    """
    Return source file extension.
    """
    return os.path.splitext(os.path.basename(codetpl_sourcefile))[1]
