#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Author: Mazhugin Aleksey
# Created: 2007/09/24
# ID: $Id: tpl_python.py 9 2007-11-20 07:31:14Z alex $
# URL: $URL: file:///H:/svn_/XPyLIB/trunc/XPyTools/codetpl/templates/tpl_python.py $
# Copyright: Copyright (c) 2007, Mazhugin Aleksey
# License: BSD
#--------------------------------------------------------------------

"""
Code templates for python.
"""

# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_class(fields):
    """
    Class template.
    """
    template= '''
class @ClassName@:
    def __init__(self):
        # Your code
    '''
    return template

# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_method(fields):
    """
    Method template.
    """
    template = '''
def @MethodName@(self):
    # Your code
    '''
    return template

# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_function(fields):
    """
    Fuction temlate.
    """
    template = '''
def @FunctionName@():
    # Your code
    '''
    return template

# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_iterator(fields):
    """
    Iterator's methods template.
    """
    template = '''
def __iter__(self):
    return self
def next(self):
    # You code
    raise StopIteration
    '''
    return template

# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_descriptor(fields):
    """
    Descriptor template.
    """
    template = '''
def __get__(self, name, owner):
    # Your code
def __set__(self, name, value):
    # Your code
def __delete__(self, name):
    # Your code
    '''
    return template

# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_xloginit(fields):
    """
    Initialize XPyLIB logging system for module.
    """
    template = '''

##:LOGGING:start
import XPyLIB.xlogging as _xlog
from functools import wraps
_log=_xlog.getLogger(__name__)
_log.disabled=0
_log_u=_xlog.IndentFormatter.lvlup
_log_d=_xlog.IndentFormatter.lvldown
_log_r=_xlog.IndentFormatter.lvlreset
_log_w=_xlog.IndentFormatter.lvlwrap
def _log_wl(fun):
    @wraps(fun)
    def wrapper(*args, **kwds):
        return _xlog.IndentFormatter.lvlwraplog(fun,_log)(*args, **kwds)
    return wrapper
##:LOGGING:end

'''
    return template

# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_getattr(fields):
    """
    __getattr__ template, that filter self attributes.
    """
    template = '''
def __getattr__(self, name):
    """
    Attribute getter.
    """
    n='_'+self.__class__.__name__
    if name.startswith(n+'__'):
        n=name[len(n):]
    else:
        n=name
    if self.__dict__.has_key(n):
        # Return self attribute.
        return self.__dict__[n]
    # Outer attribute.
    r= # Your code
    return r
'''
    return template
    
# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_classsection(fields):
    """
    Section names template (used into classes in Boa editor).
    """
    template = '''
#-------@SectionName@-------------------------------------------------------------
'''
    if not fields:
        fields={'SectionName':'SectionName'}
    else:
        if fields.has_key('SectionName') and fields['SectionName']:
            n=fields['SectionName']
            if len(n) > 70:
                n=n[:70]
            template='#' + '-'*7 + n + '-'*( 79-7-len(n) )
    return template

# language = python
# category = testing
# author = Mazhugin Aleksey
def tpl_testcase(fields):
    """
    Test Case class for unittest.
    """
    template='''
class Test@Module@(unittest.TestCase):
     """@Summary@"""

     def setUp(self):
          pass

     def tearDown(self):
          pass

     def test_Method(self):
        # Your code
'''
    return template

# language = python
# category = testing
# author = Mazhugin Aleksey
def tpl_params(fields):
    """
    Debug test for codetpl.
    
    Просто тестовый шаблон для разработчика.
    """
    template='''
'''
    return str(codetpl_params)

# language = python
# category = common
# author = Mazhugin Aleksey
def tpl_moduleheader(fields):
    """
    Header block.
    """
    if not fields:
        fields['Coding']='utf-8'
    template='''
#!/usr/bin/env python
# -*- coding: @Coding@ -*-
#--------------------------------------------------------------------
# Filname: @FileName@
# Author: @Author@
# Created: @Date@
# ID: $Id'''+''': $
# URL: $URL'''+''': $
# Copyright: @Copyright@
# License: @License@
#--------------------------------------------------------------------
'''
    return template
