#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Filname: codetpl_test.py
# Author: Mazhugin Aleksey
# Created: 20007/10/03
# ID: $Id: codetpl_test.py 21 2008-11-29 18:40:43Z alex $
# URL: $URL: file:///myfiles/svn_/XPyLIB/trunc/XPyTools/codetpl/test/codetpl_test.py $
# Copyright: (c) Mazhugin Aleksey
# License: BSD
#--------------------------------------------------------------------

"""Unit test for XPyTools.codetpl.codetpl."""

import XPyTools.codetpl.codetpl
import os
import XPyLIB.dbg as dbg ##@
import unittest

#start test

class TestCodeTemplateMngr(unittest.TestCase):
    """
    Test case for CodeTemplateMngr.
    """
    
    def setUp(self):
        ## dbg.writeln() ##@
        ## dbg.writeln( "dir='", os.path.dirname(__file__),"'" ) ##@
        self.tm = XPyTools.codetpl.codetpl.CodeTemplateMngr( \
            os.path.dirname(__file__), False )
          
    def tearDown(self):
        pass
    
    def test_Init(self):
        ## Call reload for test initial loading.
        ##self.test_Reload()
        pass

    def test_Reload(self):
        self.assertEqual(self.tm.Errors, [], 'Error while loading module.')
        ## dbg.writeln( 'Modules==', self.tm._CodeTemplateMngr__mdls ) ##@
        self.assert_('kwd_k' in self.tm._CodeTemplateMngr__mdls, \
            'kwd_k.py not loaded.')
        self.assert_('tpl_t' in self.tm._CodeTemplateMngr__mdls, \
            'tpl_t.py not loaded.')
        self.assertEqual(len(self.tm._CodeTemplateMngr__mdls), 2, \
            'Loaded modules is not equal 6 => ' + \
            str(len(self.tm._CodeTemplateMngr__mdls)) + \
            '\n\t Modules==' + str(self.tm._CodeTemplateMngr__mdls) )
            
        self.assert_('author' in self.tm._CodeTemplateMngr__kwds, \
            'Template kwd_author not find.')
        self.assert_('copyright' in self.tm._CodeTemplateMngr__kwds, \
            'Template kwd_copyright not find.')
        self.assert_('filename' in self.tm._CodeTemplateMngr__kwds, \
            'Template kwd_filename not find.')
        self.failIf('not_templatefun' in self.tm._CodeTemplateMngr__kwds, \
            'Wrong template not_templatefun is loaded.')
        self.failIf('templatefun' in self.tm._CodeTemplateMngr__kwds, \
            'Wrong template not_templatefun is loaded.')
        self.assertEqual(len(self.tm._CodeTemplateMngr__kwds), 3, \
            'Loaded modules is not equal 3 => ' + \
            str(len(self.tm._CodeTemplateMngr__kwds)) )
            
        self.assert_('tpl_class' in self.tm._CodeTemplateMngr__tpls, \
            'Template tpl_class not find.')
        self.failIf('not_templatefun' in self.tm._CodeTemplateMngr__tpls, \
            'Wrong template not_templatefun is loaded.')
        self.assertEqual(len(self.tm._CodeTemplateMngr__tpls), 1, \
            'Loaded modules is not equal 1 => ' + \
            str(len(self.tm._CodeTemplateMngr__tpls)) )
        self.assert_('kwd' in self.tm.Types, \
            'Template type "kwd" is fail.' )
        self.assert_('tpl' in self.tm.Types, \
            'Template type "tpl" is fail.' )
        self.assert_('not' not in self.tm.Types, \
            'Template type "not" is fail.' )
        self.assertEqual(len(self.tm.Types), 2, \
            'Len of Types is wrong (not 2)' )
            
        self.assertEqual(self.tm.GetConfig('Prefs','ReloadTemplates'), 'no', \
            'Load configuration fail.' )
        f=self.tm.GetConfig('Prefs','types')
        ## dbg.writeln('\n types=',f,'type== ', type(f)) ##@
        self.assertEqual(self.tm.GetConfig('Prefs','types'), \
            'act, new, wzd, kwd, tpl, wtp', \
            'Load configuration fail.' )
        pass

    def test_HasType(self):
        self.assert_(self.tm.HasType('kwd'), \
            'HasType kwd is wrong' )
        self.assert_(self.tm.HasType('tpl'), \
            'HasType tpl is wrong' )
        self.failIf(self.tm.HasType('not'), \
            'HasType not is wrong' )
        pass
        
    def test_GetTemplates(self):
        ts=self.tm.GetTemplates('kwd',{}, True)
        ## dbg.writeln(ts) ##@
        self.assert_('author' in ts, \
            'Template type "author" is fail.' )
        self.assert_('copyright' in ts, \
            'Template type "copyright" is fail.' )
        self.assert_('filename' in ts, \
            'Template type "filename" is fail.' )
        self.assert_([] == self.tm.GetTemplates('not') )
        self.assertEqual(len(ts), 3, \
            'Len of Types is wrong (not 3)' )
        
        ts=self.tm.GetTemplates('tpl',{}, True)
        self.assert_('class' in ts, \
            'Template type "class" is fail.' )
        self.assert_([] == self.tm.GetTemplates('not') )
        self.assertEqual(len(ts), 1, \
            'Len of Types is wrong (not 1)' )
        pass
    
    def test_GetMetaInfo(self):
        mi=self.tm.GetMetaInfo('kwd', 'Author')
        ##dbg.writeln('\n\t SELF.META==', self.tm._CodeTemplateMngr__tplsmeta['kwd_author']) ##@
        ##dbg.writeln('\n\t META==', mi) ##@
        self.assert_(mi[0]=='Return author.\n\nBy default get current user and return it.', \
            'Template type "author" is fail.' )
        self.assert_(mi[1]['category']=='Python', \
            'Template type "author" is fail.' )
        self.assert_(mi[1]['hotkey']=='Alt+A', \
            'Template type "author" is fail.' )
        self.assert_(mi[1]['language']=='python', \
            'Template type "author" is fail.' )
        self.assert_(mi[1]['author']=='Mazhugin Aleksey', \
            'Template type "author" is fail.' )
        pass
    
    def test_GetMetaAllValues(self):
        self.assertEqual(self.tm.GetMetaAllValues('kwd', 'hotkey'), ['Alt+A'], \
            'Meta hotkey error.')
        self.assert_('Mazhugin Aleksey' in self.tm.GetMetaAllValues('kwd', 'author'), \
            'Meta author error.')
        self.assert_('Other Mazhugin Aleksey' in self.tm.GetMetaAllValues('kwd', 'author'), \
            'Meta author error.')
        ## dbg.writeln('\n\t**** ', self.tm.GetMetaAllValues('kwd', 'author') ) ##@
        self.assertEqual(len(self.tm.GetMetaAllValues('kwd', 'author')), 2, \
            'Meta author len error.')
        pass
    
    def test_ReplaceTemplate(self):
        tpl='''
        # Copyright=@Copyright@
        # 
        class @ClassName@(@BaseClasses@):
            f=field
            ## dbg.writeln('Its debug') ##@@
            # You code
        '''
        rtpl='''
        # Copyright=(c) COPYRIGHT
        # 
        class ClassName(BaseClasses):
            f=field
            ## dbg.writeln('Its debug') ##@
            # You code
        '''
        fields={}
        kwds=None ## Use from kwd_k.py.
        ## dbg.writeln('\n') ##@
        ## dbg.writeln( '==fields=%s' % str(fields) ) ##@
        r=self.tm.ReplaceTemplate(tpl, fields, kwds, '@', False)
        ## dbg.writeln('Replace == :\n'+r) ##@
        ## dbg.writeln( '==fields=%s' % str(fields) ) ##@
        
        ## dbg.writeln('assert=%s' % (''.join(tpl.splitlines(True)[1:-1]),)) ##@
        self.assertEqual(r, ''.join(rtpl.splitlines(True)[1:-1]), \
            'Template is not parsed rightly.')
            
        rtpl='''
        # Copyright=My copyright!!!
        # 
        class ClassName(BaseClasses):
            f=field
            ## dbg.writeln('Its debug') ##@
            # You code
        '''
        fields['Copyright']='My copyright!!!'
        r=self.tm.ReplaceTemplate(tpl, fields, kwds, '@', False)
        ## dbg.writeln('Replace == :\n'+r) ##@
        ## dbg.writeln( '==fields=%s' % str(fields) ) ##@
        self.assertEqual(r, ''.join(rtpl.splitlines(True)[1:-1]), \
            'Template is not parsed rightly.')
        pass
    
    def test_ParseTemplate(self):
        rtpl='''
# Copyright=(c) COPYRIGHT
# URL=
class ClassName(BaseClasses):
    f=field
    ## dbg.writeln('Its debug') ##@
    # You code
        '''
        fields={}
        r=self.tm.ParseTemplate('tpl', 'class', fields)
        self.assertEqual(self.tm._CodeTemplateMngr__srcfile, '', \
            'Wrong source file= %s' % self.tm._CodeTemplateMngr__srcfile)
        ## dbg.writeln('Replace == :\n'+r) ##@
        ## dbg.writeln( '==fields=%s' % str(fields) ) ##@
        self.assertEqual(r, ''.join(rtpl.splitlines(True)[1:-1]), \
            'Template is not parsed rightly.')
        self.assertRaises(XPyTools.codetpl.codetpl.codetplArgumentError, \
            self.tm.ParseTemplate,'tpl', 'class', ['not dict'] )
        self.assertRaises(XPyTools.codetpl.codetpl.codetplArgumentError, \
            self.tm.ParseTemplate,'not', 'class', fields )
            
        rtpl='''
# Copyright=(c) COPYRIGHT
# URL=C:/sourcefile.py
class MyClassName(BaseClasses):
    f=field
    ## dbg.writeln('Its debug') ##@
    # You code
        '''
        fields['ClassName']='MyClassName'
        del fields['FileName']
        r=self.tm.ParseTemplate('tpl', 'class', fields, 'C:/sourcefile.py')
        self.assertEqual(self.tm._CodeTemplateMngr__srcfile, 'C:/sourcefile.py', \
            'Wrong source file= %s' % self.tm._CodeTemplateMngr__srcfile)
        ## dbg.writeln('Replace == :\n'+r) ##@
        ## dbg.writeln( '==fields=%s' % str(fields) ) ##@
        self.assertEqual(r, ''.join(rtpl.splitlines(True)[1:-1]), \
            'Template is not parsed rightly.')
        pass
    
#end test

if __name__=='__main__':
    unittest.main(argv=['','-v'])
