#!/usr/bin/env python
# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Filname: pep_parse.py
# Author: Mazhugin Aleksey
# Created: 2007/11/29
# ID: $Id: $
# URL: $URL: $
# Copyright: Copyright (c) 2007, Mazhugin Aleksey.
# License: BSD
#--------------------------------------------------------------------

"""
Rename PEPs html and generate index page "pep_index.html".

Read folder contens (*.html files) and if they like PEP html then
rename they to "PEP 0000 -- Title". Escape all not filename symbols ("*?:/\\").
"""

# Importing
##import XPyLIB.dbg as dbg ##@
import XPyLIB
import re
import sys
import glob
import getopt
import os
import string

##:LOGGING:start
import XPyLIB.xlogging as _xlog
from functools import wraps
_log=_xlog.getLogger('XPyTools.scripts.pep_parser') ##__name__)
_log.disabled=1
_log_u=_xlog.IndentFormatter.lvlup
_log_d=_xlog.IndentFormatter.lvldown
_log_r=_xlog.IndentFormatter.lvlreset
_log_w=_xlog.IndentFormatter.lvlwrap
def _log_wl(fun):
    @wraps(fun)
    def wrapper(*args, **kwds):
        return _xlog.IndentFormatter.lvlwraplog(fun,_log)(*args, **kwds)
    return wrapper
##:LOGGING:end

_log.debug('logging enabled')

# Start Implementation

def fail(fn='', errstr='Error.'):
    """
    Fail reporting.
    """
    print 'FAIL: file="%s". %s\n' % (fn, errstr)

def usage():
    """
    Output help info.
    """
    print r'''
    Usage:
        i, indir=    - input dir for pep *.html files.
        o, outdir=   - output dir, copy renamed pep files and create index.html file.
        h,?, help      - output this help info.
    '''

def do():
    """
    Pars peps.
    """
    
    # Parse options.
    opt = 'i:o:h?'
    lopt = ['indir=', 'outdir=', 'help']
        
    try:
        opts, args = getopt.getopt(sys.argv[1:], opt, lopt)
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)

    indir = os.getcwdu()
    outdir = os.path.join(indir, 'peps')

    for o,v in opts:
        if o in ('?','-h','--help'):
            usage()
            sys.exit(0)
        elif o in ('-i', '--indir'):
            indir=v
        elif o in ('-o', '--outdir'):
            outdir=v
        else:
            assert False, "unhandled option" ##@@

    if not os.path.exists(indir):
        print 'Input dir "%s" not exist.' % (indir,)
        sys.exit(2)

    if not os.path.exists(outdir):
        try:
            os.makedirs(outdir)
        except:
            print 'Output dir "%s" not exist, and can\'t be created.' % (outdir,)
            sys.exit(2)
            
    re_pep = re.compile(r'<tr["=\w -]*>\s*<th["=\w \-]*>PEP:[&nbsp; ]*</th>\s*<td["=\w \-]*>(?P<pepid> *\d+ *)</td>\s*</tr>\s*<tr["=\w \-]*><th["=\w \-]*>Title:[&nbsp; ]*</th>\s*<td["=\w \-]*>(?P<peptitle>[\S ]+)</td>\s*</tr>')

    tt=string.maketrans(':\\/*+?\'"&<>','           ')
    
    _log.debug('pattern=%s' % (os.path.join(indir,'*.htm'),) )
    fl = glob.glob(os.path.join(indir,'*.htm'))
    _log.debug('fl=%s' % (str(fl),) )
    # Index for files: [(Id, Title, path)].
    idx = []

    # Main loop.
    for fn in fl:
        _log.debug( 'try %s\n' % (fn,) )
        try:
            f=open(fn,'r')
            ln=f.read()
        except Exception, err:
            fail( fn, 'READ EXCEPTION=%s.' % (str(err),) )
            if f:
                f.close()
                f=None
            continue
        f.close()
        f=None
        
        s = re_pep.search(ln)
        if not s:
            _log.debug('fn search fail\n')
            continue
        _log.debug('fn search ok\n')
        d = s.groupdict()
        if not ( d.has_key('pepid') and d.has_key('peptitle') ):
            fail(fn, 'No PEP id and/or title.\n')
        try:
            id=int(d['pepid'])
        except:
            fail( fn, 'Wrong PEP id="%s".\n' % (d['pepid'],) )
            continue
        title=d['peptitle'].strip()
        tn=title.translate(tt)
        name='PEP %d -- %s.html' % (id, tn )
        p=os.path.join(outdir, name)
        idx.append((id, title, name))
        _log.debug('try write path: %s\n' % (p,) )
        try:
            f=open(p,'w')
            f.write(ln)
            _log.debug('\nwritten=%s\n' % (ln[:256],))
        except Exception, err:
            fail( fn, 'WRITE EXCEPTION=%s.' % (str(err),) )
            if f:
                f.close()
                f=None
            continue
        f.close()
        f=None
        
    ##end for fn

    _log.debug('generate index.html\n')
    
    ois=r'''<html>
    <head>
    <title> PEPs index</title>
    </head>

    <body>
    <table border=1><thead><th>Id</th><th>Title</th></thead>
    <tbody>'''

    oie=r'''</tbody></table>
    </body>
    </html>
    '''

    idx=sorted(idx, lambda x,y: cmp(x[0],y[0]) )
    htm=ois
    for i in idx:
        htm+='<tr>\n<td><a href="%s">%d</a></td>\n<td>%s</td>\n</tr>\n' % (i[2],i[0],i[1])

    htm+=oie

    try:
        f=open(os.path.join(outdir, 'index.html'),'w')
        f.write(htm)
    except Exception, err:
        print 'WRITE INDEX EXCEPTION=%s.' % (str(err),) 
    if f:
        f.close()


##end def do

# End Implemetation

#-DEBUG-START-------------------------------------------------------------------

def _debug():
    """
    """
    ## _log.debug('Debug message')
    pass
    
#-DEBUG-END---------------------------------------------------------------------

if __name__=='__main__':
    try:
        do()
    except Exception, err:
        print 'ERROR=%s.' % (str(err),)
    else:
        print '\nWork complete.\n'
        
